@@ -15,8 +15,12 @@
  * #GsHardwareSupportContextDialog is a dialog which shows detailed information
  * about what hardware an app requires or recommends to be used when running it.
  * For example, what input devices it requires, and what display sizes it
- * supports. This information is derived from the `<requires>` and
- * `<recommends>` elements in the app’s appdata.
+ * supports. This information is derived from the `<requires>`,
+ * `<recommends>` and `<supports>` elements in the app’s appdata.
+ *
+ * Currently, `<supports>` is treated as a synonym of `<recommends>` as it’s
+ * only just been introduced into the appstream standard, and many apps which
+ * should be using `<supports>` are still using `<recommends>`.
  *
  * It is designed to show a more detailed view of the information which the
  * app’s hardware support tile in #GsAppContextBar is derived from.
@@ -29,33 +33,33 @@
 
 #include "config.h"
 
+#include <adwaita.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <handy.h>
 #include <locale.h>
 
 #include "gs-app.h"
 #include "gs-common.h"
 #include "gs-context-dialog-row.h"
 #include "gs-hardware-support-context-dialog.h"
+#include "gs-lozenge.h"
 
 struct _GsHardwareSupportContextDialog
 {
-	HdyWindow		 parent_instance;
+	GsInfoWindow		 parent_instance;
 
 	GsApp			*app;  /* (nullable) (owned) */
 	gulong			 app_notify_handler_relations;
 	gulong			 app_notify_handler_name;
 
-	GtkImage		*icon;
 	GtkWidget		*lozenge;
 	GtkLabel		*title;
 	GtkListBox		*relations_list;
 };
 
-G_DEFINE_TYPE (GsHardwareSupportContextDialog, gs_hardware_support_context_dialog, HDY_TYPE_WINDOW)
+G_DEFINE_TYPE (GsHardwareSupportContextDialog, gs_hardware_support_context_dialog, GS_TYPE_INFO_WINDOW)
 
 typedef enum {
 	PROP_APP = 1,
@@ -127,6 +131,9 @@ add_relation_row (GtkListBox                   *list_box,
 		}
 		break;
 	case AS_RELATION_KIND_RECOMMENDS:
+#if AS_CHECK_VERSION(0, 15, 0)
+	case AS_RELATION_KIND_SUPPORTS:
+#endif
 		rating = GS_CONTEXT_DIALOG_ROW_IMPORTANCE_UNIMPORTANT;
 		icon_name = icon_name_recommends;
 		title = title_recommends;
@@ -143,7 +150,7 @@ add_relation_row (GtkListBox                   *list_box,
 		*chosen_rating = rating;
 
 	row = gs_context_dialog_row_new (icon_name, rating, title, description);
-	gtk_list_box_insert (list_box, GTK_WIDGET (row), -1);
+	gtk_list_box_append (list_box, GTK_WIDGET (row));
 }
 
 /**
@@ -161,17 +168,20 @@ add_relation_row (GtkListBox                   *list_box,
 GdkMonitor *
 gs_hardware_support_context_dialog_get_largest_monitor (GdkDisplay *display)
 {
+	GListModel *monitors;  /* (unowned) */
 	GdkMonitor *monitor;  /* (unowned) */
-	int n_monitors, monitor_max_dimension;
+	int monitor_max_dimension;
+	guint n_monitors;
 
 	g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
 
-	n_monitors = gdk_display_get_n_monitors (display);
+	monitors = gdk_display_get_monitors (display);
+	n_monitors = g_list_model_get_n_items (monitors);
 	monitor_max_dimension = 0;
 	monitor = NULL;
 
-	for (int i = 0; i < n_monitors; i++) {
-		GdkMonitor *monitor2 = gdk_display_get_monitor (display, i);
+	for (guint i = 0; i < n_monitors; i++) {
+		g_autoptr(GdkMonitor) monitor2 = g_list_model_get_item (monitors, i);
 		GdkRectangle monitor_geometry;
 		int monitor2_max_dimension;
 
@@ -181,9 +191,7 @@ gs_hardware_support_context_dialog_get_largest_monitor (GdkDisplay *display)
 		gdk_monitor_get_geometry (monitor2, &monitor_geometry);
 		monitor2_max_dimension = MAX (monitor_geometry.width, monitor_geometry.height);
 
-		if (monitor2_max_dimension > monitor_max_dimension ||
-		    (gdk_monitor_is_primary (monitor2) &&
-		     monitor2_max_dimension == monitor_max_dimension)) {
+		if (monitor2_max_dimension > monitor_max_dimension) {
 			monitor = monitor2;
 			monitor_max_dimension = monitor2_max_dimension;
 			continue;
@@ -204,6 +212,10 @@ max_relation_kind (AsRelationKind kind1,
 		return AS_RELATION_KIND_REQUIRES;
 	if (kind1 == AS_RELATION_KIND_RECOMMENDS || kind2 == AS_RELATION_KIND_RECOMMENDS)
 		return AS_RELATION_KIND_RECOMMENDS;
+#if AS_CHECK_VERSION(0, 15, 0)
+	if (kind1 == AS_RELATION_KIND_SUPPORTS || kind2 == AS_RELATION_KIND_SUPPORTS)
+		return AS_RELATION_KIND_SUPPORTS;
+#endif
 	return AS_RELATION_KIND_UNKNOWN;
 }
 
@@ -327,6 +339,9 @@ gs_hardware_support_context_dialog_get_control_support (GdkDisplay     *display,
 			control_relations[control_kind] = MAX (control_relations[control_kind], kind);
 
 			if (kind == AS_RELATION_KIND_REQUIRES ||
+#if AS_CHECK_VERSION(0, 15, 0)
+			    kind == AS_RELATION_KIND_SUPPORTS ||
+#endif
 			    kind == AS_RELATION_KIND_RECOMMENDS)
 				any_control_relations_set = TRUE;
 		}
@@ -392,8 +407,9 @@ gs_hardware_support_context_dialog_get_control_support (GdkDisplay     *display,
  *
  * @desktop_relation_kind_out is set to the type of support the app has for
  * desktop displays: whether they’re required (%AS_RELATION_KIND_REQUIRES),
- * supported but not required (%AS_RELATION_KIND_RECOMMENDS) or whether there’s
- * no information (%AS_RELATION_KIND_UNKNOWN).
+ * supported but not required (%AS_RELATION_KIND_RECOMMENDS or
+ * %AS_RELATION_KIND_SUPPORTS) or whether there’s no information
+ * (%AS_RELATION_KIND_UNKNOWN).
  *
  * @mobile_match_out and @mobile_relation_kind_out behave similarly, but for
  * mobile displays (smaller than 768 pixels).
@@ -515,7 +531,7 @@ update_relations_list (GsHardwareSupportContextDialog *self)
 	 * support based on app properties. */
 	chosen_rating = GS_CONTEXT_DIALOG_ROW_IMPORTANCE_NEUTRAL;
 
-	gs_container_remove_all (GTK_CONTAINER (self->relations_list));
+	gs_widget_remove_all (GTK_WIDGET (self->relations_list), (GsRemoveFunc) gtk_list_box_remove);
 
 	/* UI state is undefined if app is not set. */
 	if (self->app == NULL)
@@ -697,7 +713,7 @@ update_relations_list (GsHardwareSupportContextDialog *self)
 		css_class = "grey";
 		break;
 	case GS_CONTEXT_DIALOG_ROW_IMPORTANCE_UNIMPORTANT:
-		icon_name = "test-pass-symbolic";
+		icon_name = "app-installed-symbolic";
 		/* Translators: The app will work on the current hardware.
 		 * The placeholder is the app name. */
 		title = g_strdup_printf (_("%s works on this device"), gs_app_get_name (self->app));
@@ -721,7 +737,7 @@ update_relations_list (GsHardwareSupportContextDialog *self)
 		g_assert_not_reached ();
 	}
 
-	gtk_image_set_from_icon_name (GTK_IMAGE (self->icon), icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
+	gs_lozenge_set_icon_name (GS_LOZENGE (self->lozenge), icon_name);
 	gtk_label_set_text (self->title, title);
 
 	context = gtk_widget_get_style_context (self->lozenge);
@@ -744,35 +760,6 @@ app_notify_cb (GObject    *obj,
 	update_relations_list (self);
 }
 
-static gboolean
-key_press_event_cb (GtkWidget            *sender,
-                    GdkEvent             *event,
-                    HdyPreferencesWindow *self)
-{
-	guint keyval;
-	GdkModifierType state;
-	GdkKeymap *keymap;
-	GdkEventKey *key_event = (GdkEventKey *) event;
-
-	gdk_event_get_state (event, &state);
-
-	keymap = gdk_keymap_get_for_display (gtk_widget_get_display (sender));
-
-	gdk_keymap_translate_keyboard_state (keymap,
-					     key_event->hardware_keycode,
-					     state,
-					     key_event->group,
-					     &keyval, NULL, NULL, NULL);
-
-	if (keyval == GDK_KEY_Escape) {
-		gtk_window_close (GTK_WINDOW (self));
-
-		return GDK_EVENT_STOP;
-	}
-
-	return GDK_EVENT_PROPAGATE;
-}
-
 static void
 gs_hardware_support_context_dialog_init (GsHardwareSupportContextDialog *self)
 {
@@ -854,12 +841,9 @@ gs_hardware_support_context_dialog_class_init (GsHardwareSupportContextDialogCla
 
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Software/gs-hardware-support-context-dialog.ui");
 
-	gtk_widget_class_bind_template_child (widget_class, GsHardwareSupportContextDialog, icon);
 	gtk_widget_class_bind_template_child (widget_class, GsHardwareSupportContextDialog, lozenge);
 	gtk_widget_class_bind_template_child (widget_class, GsHardwareSupportContextDialog, title);
 	gtk_widget_class_bind_template_child (widget_class, GsHardwareSupportContextDialog, relations_list);
-
-	gtk_widget_class_bind_template_callback (widget_class, key_press_event_cb);
 }
 
 /**
